"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@babel/types");
const convert_flow_type_1 = require("../converters/convert_flow_type");
const convert_type_parameter_declaration_1 = require("../converters/convert_type_parameter_declaration");
function convert_interface_declaration(path) {
    const origBody = path.get('body');
    const origExtends = path.get('extends');
    const origImplements = path.get('implements');
    const origTypeParameters = path.get('typeParameters');
    let origExtendsCombined = [];
    if (Array.isArray(origExtends)) {
        origExtendsCombined = origExtendsCombined.concat(origExtends);
    }
    if (Array.isArray(origImplements)) {
        origExtendsCombined = origExtendsCombined.concat(origImplements);
    }
    const typeParameters = origTypeParameters.node
        ? convert_type_parameter_declaration_1.convertTypeParameterDeclaration(origTypeParameters)
        : null;
    const members = [];
    const extending = [];
    origBody.node.properties.forEach((property, i) => {
        if (types_1.isObjectTypeProperty(property)) {
            const tsPropSignature = types_1.tsPropertySignature(property.key, types_1.tsTypeAnnotation(convert_flow_type_1.convertFlowType(origBody.get(`properties.${i}.value`))));
            tsPropSignature.optional = property.optional;
            tsPropSignature.readonly = property.variance && property.variance.kind === 'plus';
            members.push(tsPropSignature);
        }
    });
    origExtendsCombined.forEach(origExtend => {
        const origExtendTypeParameters = origExtend.get('typeParameters');
        const origExtendTypeParametersParams = origExtendTypeParameters.node
            ? origExtendTypeParameters.get('params')
            : [];
        const parameters = types_1.tsTypeParameterInstantiation(origExtendTypeParametersParams.map(item => convert_flow_type_1.convertFlowType(item)));
        extending.push(types_1.tsExpressionWithTypeArguments(origExtend.node.id, origExtendTypeParametersParams.length ? parameters : null));
    });
    return types_1.tsInterfaceDeclaration(path.node.id, typeParameters, extending.length ? extending : null, types_1.tsInterfaceBody(members));
}
exports.convert_interface_declaration = convert_interface_declaration;
//# sourceMappingURL=convert_interface_declaration.js.map